#CMRPopHet version: 2.2.2
#By: S.J. Puechmaille & Petit E.J.
#Date: 27/03/2011
#Please, report any suggestions to improve the script, input or output files to: s [dot] puechmaille [at] gmail [dot] com

#####################################
#	Population size estimate	#
#	Petit and Valire, 2006		#
#####################################

CMRpopsize=function(name,samples,indiv,maxPop,graph="TRUE",plotNvalue="Mean",plotLimit="HPD95")	{

#Formats the data
data=matrix(0,samples,4)
colnames(data)=c("t","Ct","Mt","Rt")
data[,1]=seq(1,samples,by=1)
data[,2]=1
data[1:indiv,3]=seq(1,indiv,by=1)
data[(indiv+1):samples,3]=data[indiv,3]
data[1:indiv,4]=0
data[(indiv+1):samples,4]=1

#Maximum Population size
NK=maxPop
    
head(data)
#initialization
    moy = 0
    mod = 0
    modpr = 0
    med = 0
    medpr = 0
    quantile1 = 0
    quantile2 = 0
    hpd1 = 0
    hpd2 = 0
    C = data[,2]
    M = data[,3]
    R = data[,4]
    
#the minimum population size is set to the number of genotypes detected
    N0 = max(M)
    
#we evaluate all population sizes between and including N0 and NK
    nK = NK - N0 + 1
    
#objects for N and P
    N = array(0, dim = nK)
    P = array(0, dim = c(samples + 1, nK))
    
#set the lower and upper bounds and the number of points to evaluate
	N=seq(indiv,maxPop,by=1); P[1,]=1
    
#recursive process to calculate the posterior probability of each N tested
    for (t in 2:(samples + 1)) {
        for (i in 1:nK) {
            P[t, i] = (M[t - 1]/N[i])^R[t - 1] * ((1 - (M[t - 
                1]/N[i]))^(C[t - 1] - R[t - 1])) * P[t - 1, i]
        }
    }
    
#normalization
    SumP = array(0, dim = (samples + 1))
    for (t in 1:(samples + 1)) {
            SumP[t] = SumP[t] + sum(P[t, ])
        }
	P[2:(samples+1),]=apply(P[2:(samples+1),],2,function(x) (x/t(SumP[2:(samples+1)])))

#calculate the mean
	moy=sum(N*P[samples+1,])
    
#calculate the median
	med2=0
    	for (i in 1:nK) 
      	med2[i] = sum(P[samples + 1, 1:i])
		med=N[which(abs(med2-0.5)==min(abs(med2-0.5)))]

#calculate mode
	mod = 0
	mod=N[match(max(P[samples + 1,]),P[samples + 1,],nomatch=0)]

#calculate the 2.5% and 97.5% quantiles
	quantile1=N[which(abs(med2-0.025)==min(abs(med2-0.025)))]
	quantile2=N[which(abs(med2-0.975)==min(abs(med2-0.975)))]
   
#calculate the Highest Probability Density (HPD)
    hpd1 = N0
    hpd2 = NK
    quant = array(0, dim = c(nK, nK))
    for (i in 1:nK) {
        for (j in i:nK) {
            quantile2bis = sum(P[samples + 1, i:j])
            if (quantile2bis >= 0.95 && N[j] - N[i] < hpd2 - hpd1) {
                hpd1 = N[i]
                hpd2 = N[j]
            }
        }
    }
    
#Starts saving the pdf "name_CMR-Pop.pdf" if graph="TRUE" (for single data set only)
	if (graph=="TRUE")
	pdf(file=paste("CMR-Pop_Graph_",name,"_",plotNvalue,"_",plotLimit,".pdf",sep=""),width=12,height=8)

#plot the probability functions
	#First set the X-axis max coordinate
	xl=N[match(c(0.999+min(abs(med2-0.999)),0.999-min(abs(med2-0.999))),med2,nomatch=0)]
	plot(c(N0:NK), P[length(M), ],xlim=c(indiv,xl),  type = "l", xlab = "Ni", ylab = "Posterior probability",cex.main=0.8,main=paste(name,", ", samples, " samples, ", indiv, " individuals; Mode-Median-Mean=",mod,"-",med,"-",round(moy,0),", HPD 95% [",hpd1,"-",hpd2,"], IC 95% [",quantile1,"-",quantile2, "], range of N considered [",indiv,"-",maxPop,"]",sep=""))
	if (plotNvalue=="Median")
	abline(v=med,lty=1,lwd=2)
	if (plotNvalue=="Mean")
	abline(v=moy,lty=1,lwd=2)
	if (plotNvalue=="Mode")
	abline(v=mod,lty=1,lwd=2)
	if (plotLimit=="HPD95")
	abline(v=c(hpd1,hpd2),lty=2,lwd=2)
	if (plotLimit=="IC95")
	abline(v=c(quantile1,quantile2),lty=2,lwd=2)
		
#Starts saving the pdf "name_CMR-Pop.pdf"
	if (graph=="TRUE")
	dev.off()

#the results
    cat("Population:", name ,"\nMode-Median-Mean:        ", mod,"-",med,"-",round(moy,0), "\nQuantile 95%: [", quantile1, ";", quantile2, "]\nHPD 95%:      [", hpd1, ";", hpd2, "]\n") 

	results=matrix(0,1,12)
	colnames(results)=c("Pop name","N. samples","N. indiv","Max Popsize","Mode","Median","Mean","2.5% quantile","97.5% quantile","2.5% HPD","97.5% HPD","date-time of run")
	results[1,1]=name
	results[1,2]=samples
	results[1,3]=indiv
	results[1,4]=maxPop
	results[1,5]=mod
	results[1,6]=med
	results[1,7]=round(moy,0)
	results[1,8]=quantile1
	results[1,9]=quantile2
	results[1,10]=hpd1
	results[1,11]=hpd2
	results[1,12]=format(Sys.time(), "%d-%b-%Y %H:%M:%S")
	#colnames(results)=c("Samples","Individuals","maxPop","Mode","Quantile 2.5%","Quantile 97.5%","HPD 95% low","HPD 95% high","Date")
	write.table(results,file="CMR-Pop_Results.txt",append=TRUE,quote=FALSE,row.names=FALSE,col.names=FALSE,sep = "\t")
}

#For multiple datasets:

	CMRpopsizeM=function(input,plotNvalue="Mean",plotLimit="HPD95") {
	datat=read.table(input)
	datat$V1=as.factor(datat$V1)
	len=nrow(datat)
#Starts saving the pdf "CMR-Pop_Graph.pdf"
	pdf(file=paste("CMR-Pop_Graph_",plotNvalue,"_",plotLimit,".pdf",sep=""),width=12,height=8)
	
	for (i in 1:len)	{
	CMRpopsize(name=paste(datat[i,1]),samples=datat[i,2],indiv=datat[i,3],maxPop=datat[i,4],graph="FALSE",plotNvalue,plotLimit)	}

#Stops saving the pdf "CMR-Pop_Graph.pdf"
	dev.off()
		}

#########################################
#	    Heterogeneity test		#
#	Puechmaille & Petit, 2007	#
#########################################

CMRpophet=function(name,obser,samples,popsize,replicates,alpha=0.05)	{

#Observed data
obs=obser
lep=length(obs)
#Estimated popualtion size (i.e. obtained with the 'CMRpopsize' function)
N=popsize
#Number of samples (droppings analysed)
n=samples
#Number of replicates
t=replicates
#A bag with all individuals
sac=c(1:N)
#A matrix where the results of the t replicates will be stored
res=matrix(0,lep,t)
for (i in 1:t) {
	#n random are drawn with replacment
	tirage=sample(sac,n,replace=TRUE)
	#Table giving the number of capture per individual 
	ncapt=as.data.frame(table(tirage))
	#Table giving the number of individual captured once, twice, etc.
	freq=as.data.frame(table(ncapt$Freq))
	freq$Var1=as.numeric(as.character(freq$Var1))
	#Matrix containing all the results 
	for (j in 1:lep) {
		for (k in 1:length(freq$Var1)) {
			if (freq[k,1]==j)
			res[j,i]=freq[k,2]
		}
	}
}

#Analisis of the results
moy=numeric(lep)
CIsup=numeric(lep)
CIinf=numeric(lep)
higIC=1-alpha/2
lowIC=0+alpha/2
for (i in 1:lep) {
	moy[i]=mean(res[i,])
	tri=sort(res[i,])
	CIsup[i]=tri[round(higIC*t)]
	CIinf[i]=tri[round(lowIC*t)]
}
	#Starts saving the pdf "CMR-Pop.pdf"
pdf(file=paste("CMR-Het_Graph_",name,"_alpha=",alpha,".pdf",sep=""),width=12,height=8)

#Plots the data
plot(CIsup,pch=1,xlab="N. of capture/individual",ylab="N. of individuals",
main=c("Heterogeneity test","Puechmaille & Petit, 2007",name))
points(moy,pch=19)
points(obs,pch=17)
points(CIinf,pch=1)
for (i in 1:lep) {
	segments(i,moy[i],i+1,moy[i+1],lty=17)
	segments(i,CIsup[i],i+1,CIsup[i+1],lty=2)
	segments(i,CIinf[i],i+1,CIinf[i+1],lty=2)
	segments(i,obs[i],i+1,obs[i+1],lty=1)
	}

dev.off()

}

#########################################
#      For multiple datasets:		#
#########################################

CMRpophetM=function (inputt,alpha=0.05)
{	
#CREATES the same function as 'CMRpophet' but for multiple datasets
CMRpophetMm=function(name,obser,samples,popsize,replicates)	{

	#Observed data
obs=obser
lep=length(obs)
	#Estimated popualtion size (i.e. obtained with the 'CMRpopsize' function)
N=popsize
	#Number of samples (droppings analysed)
n=samples
	#Number of replicates
t=replicates
	#A bag with all individuals
sac=c(1:N)
	#A matrix where the results of the t replicates will be stored
res=matrix(0,lep,t)
for (i in 1:t) {
	#n random are drawn with replacment
	tirage=sample(sac,n,replace=TRUE)
	#Table giving the number of capture per individual 
	ncapt=as.data.frame(table(tirage))
	#Table giving the number of individual captured once, twice, etc.
	freq=as.data.frame(table(ncapt$Freq))
	freq$Var1=as.numeric(as.character(freq$Var1))
	#Matrix containing all the results 
	for (j in 1:lep) {
		for (k in 1:length(freq$Var1)) {
			if (freq[k,1]==j)
			res[j,i]=freq[k,2]
		}
	}
}

	#Analisis of the results
moy=numeric(lep)
CIsup=numeric(lep)
CIinf=numeric(lep)
higIC=1-alpha/2
lowIC=0+alpha/2
for (i in 1:lep) {
	moy[i]=mean(res[i,])
	tri=sort(res[i,])
	CIsup[i]=tri[round(higIC*t)]
	CIinf[i]=tri[round(lowIC*t)]
}

	#Builts a matrix with results
resu=matrix(0,5,lep)
resu[1,]=CIsup
resu[2,]=obs
resu[3,]=CIinf
resu[4,]=moy

for (i in 1:lep) 
if (resu[1,i]>= resu[2,i] & resu[2,i]>=resu[3,i]) {
	resu[5,i]=0 
	}else{ resu[5,i]=1
}

return(resu)
}
	#END of the function 'CMRpophetMm' for multiple datasets

	#NOW reads the input data
dat=read.table(inputt)
dat$V1=as.factor(dat$V1)
numrow=nrow(dat)
numcol=ncol(dat)
	#CREATES a data frame to put the results of the test
mar=matrix(0,5*numrow,ncol(dat))
matt=as.data.frame(mar)
colnames(matt)=c("Name","Samples","Popsize","Simulations",paste("Cap",seq(1,numcol-4,by=1),sep=""))
for (i in 1:numrow)	
for (j in 1:4) {
		#APPLIES the function for mutiple datasets and puts the results in the objects 'ret'
	ret=CMRpophetMm(name=paste(dat[i,1]),obser=as.numeric(dat[i,5:ncol(dat)]),samples=dat[i,2],popsize=dat[i,3],replicates=dat[i,4])	
	#FILLS the data frame 'matt' with the results and information about samples, individuals, etc.
matt[i*5-4,j]=paste(dat[i,j])
matt[i*5-3,j]=paste(dat[i,j])
matt[i*5-2,j]=paste(dat[i,j])
matt[i*5-1,j]=paste(dat[i,j])
matt[i*5,j]=paste(dat[i,j])
#INTEGRATES 'ret' into 'matt'
matt[(i*5-4):(i*5),5:numcol]=ret[]
}

	#Calculates the max number of recapture (real or simulated) per data set to include in the graph
maxrec=matrix(numcol,numrow,2)
for (i in 1:numrow) {
	for (j in 5:numcol){
		tot=sum(dat[i,5:j])
			if(tot==sum(dat[i,5:numcol]) && j<maxrec[i,1]) 
			maxrec[i,1]<-j 
		tot=sum(matt[(i*5-4),5:j])
			if(tot==sum(matt[(i*5-4),5:numcol]) && j<maxrec[i,2]) 
			maxrec[i,2]<-j 
		}
	}
recap=maxrec-3

	#CREATES another matrix to put the summary results
resul=matrix(0,numrow,3)
colnames(resul)=c("Name","Result","Number of value outside IC95%")
resul[,1]=paste(dat[,1])

resul[,3]=paste(dat[,1])
for (i in 1:numrow)
	#TESTS if the observed value are between the IC95%
if (sum(matt[(i*5),5:numcol])==0) {
	resul[i,2]<-"Homogene"	
	resul[i,3]<-0
	}else{ 
	resul[i,2]="Heterogene"
	resul[i,3]=sum(matt[(i*5),5:numcol])
	}

	#Starts saving the pdf "CMR-Het.pdf"
pdf(file=paste("CMR-Het_Graph_alpha=",alpha,".pdf",sep=""),width=12,height=8)
	#plots the results
der=seq(1,numrow*5,by=5)
for (i in der) {
 ymax=max(t(matt[i:(i+2),5:numcol]))
 xmax=max(recap[(i+4)/5,])
 plot(t(matt[i,5:numcol]),type="l",xlim=c(1,xmax),ylim=c(0,ymax),,xlab="N. of capture/individual",ylab="N. of individuals",
main=paste(matt[i,1],", IC=",100-alpha*100,"%; Sampling ",resul[((i+4)/5),2])); points(t(matt[i,5:numcol]),pch=1)
points(t(matt[i+1,5:numcol]),type="l"); points(t(matt[i+1,5:numcol]),pch=17)
points(t(matt[i+2,5:numcol]),type="l"); points(t(matt[i+2,5:numcol]),pch=1)
points(t(matt[i+3,5:numcol]),type="l"); points(t(matt[i+3,5:numcol]),pch=16)
	}

	#Stops saving the pdf "CMR-Het_Graph.pdf"
dev.off()

	#OUTPUTS the results as text files
write.csv(matt,paste("CMR-Het_Results_alpha=",alpha,".csv",sep=""), row.names=F,quote=FALSE)
write.csv(resul,paste("CMR-Het_Summary_alpha=",alpha,".csv",sep=""),row.names=F,quote=FALSE)
	}